package example.energy.coolemall;

import schedframe.events.scheduling.EventReason;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Node;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;
import schedframe.resources.devices.Device;
import schedframe.resources.devices.Fan;
import schedframe.resources.devices.PhysicalResource;
import schedframe.scheduling.manager.tasks.JobRegistry;
import example.energy.BaseEnergyEstimationPlugin;

public class FanEnergyEstimationPlugin extends BaseEnergyEstimationPlugin {

	public double estimatePowerConsumption(EnergyEvent event, JobRegistry jobRegistry,
			PhysicalResource resource) {
		
		double powerConsumption = 0;
		
		Fan fan = (Fan) resource;
		try {
			powerConsumption = fan.getAirflowInterface().getPowerConsumption(fan.getAirflowInterface().getAirflowState());
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/*if(resource.getAirThroughputInterface().getAirThroughputState().getName().equals(new UserAirThroughputStateName("1").getName())){
			powerConsumption = CoolEmAllTestbedMeasurements.SINGLE_FAN_OFF_POWER_CONSUMPTION;
		} else {
			powerConsumption = CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_POWER_CONSUMPTION;
		}*/
		return powerConsumption * getNumberOfWorkingNodes(fan)/(double)fan.getChilledResources().size();
	}
	
	public double estimateAirflow(EnergyEvent event, JobRegistry jobRegistry, PhysicalResource resource) {
		
		double airflow = 0;
		Fan fan = (Fan) resource;
		try {
			if(event.getReason() == EventReason.SIM_INIT)
				airflow = fan.getAirflowInterface().getAirflow(fan.getAirflowInterface().getAirflowState());
			else
				airflow = fan.getAirflowInterface().getAirflow(fan.getAirflowInterface().getAirflowState());
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/*if(event.getReason() == EventReason.SIM_INIT){
			if(resource.getAirThroughputInterface().getAirThroughputState().getName().equals(new UserAirThroughputStateName("1").getName())){
				airThroughput = CoolEmAllTestbedMeasurements.SINGLE_FAN_OFF_AIR_FLOW;
			} else {
				airThroughput = CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_AIR_FLOW;
			}
		}
		else {
			if(resource.getAirThroughputInterface().getAirThroughputState().getName().equals(new UserAirThroughputStateName("1").getName())){
				airThroughput = CoolEmAllTestbedMeasurements.SINGLE_FAN_OFF_AIR_FLOW;
			} else {
				airThroughput = CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_AIR_FLOW;
			}
		}*/

		
		return airflow * getNumberOfWorkingNodes(fan)/(double)fan.getChilledResources().size();
	}
	
	private int getNumberOfWorkingNodes(Device device){
		int quantity = 0;
		Fan fan = (Fan) device;
		for(ComputingResource compRes: device.getComputingResource().getChildren()){
			Node node = (Node)compRes;
			if(fan.getChilledResources().contains(node.getFullName()) && node.getPowerInterface().getPowerState().equals(StandardPowerStateName.ON) ){
				quantity++;
			}
		}
		return quantity;
	}

}
