package example.localplugin.coolemall.computebox1;

import java.util.List;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.computing.Node;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.manager.tasks.JobRegistryImpl;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import example.localplugin.BaseLocalSchedulingPlugin;
import gridsim.dcworms.DCWormsTags;

public class CB1_FCFS_LoadBalancing_SP extends BaseLocalSchedulingPlugin {

	public CB1_FCFS_LoadBalancing_SP () {
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			 ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			
			// check all tasks in queue
			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					String nodeName = chooseProvider(resourceManager);
					if (nodeName != null) {
						addToSchedulingPlan(plan, task, nodeName);
					}
				}
			}
			break;
		}
		return plan;
	}

	private String chooseProvider(ClusterResourceManager resourceManager) {
		List<Node> nodes = resourceManager.getNodes();
		int nodeIdx = findLeastLoadedResourceIdx(nodes);
		return nodes.get(nodeIdx).getFullName();
	}
	
	private int findLeastLoadedResourceIdx(List<Node> nodes ){
		int resourceIdx = -1;
		int minLoad = Integer.MAX_VALUE;
		
		for(int i = 0; i < nodes.size(); i++){
			Node node = nodes.get(i);
			JobRegistry jr = new JobRegistryImpl(node.getFullName());
			int totalLoad = jr.getRunningTasks().size(); 
			if(totalLoad < minLoad){
				resourceIdx = i;
				minLoad = totalLoad;
			}
		}
		return resourceIdx;
	}

}

